/**
 * Introduction
 *
 * @title ROVLib - Easy applications from C
 *
 * @copyright Jason Tribbeck / 7th ARM 1994-2001
 *
 * @authors Jason Tribbeck
 *
 * @file baricon.h
 *
 * @type subsystem
 *
 * @description
 *
 * The baricon subsystem maintains an icon on the icon bar. It is a much simpler
 * system to use than the main icon system, but this is more limited. It should
 * be used to create one icon on the icon bar.
 *
 * When created, the icon has a 'click' handler, which is called when the user
 * clicks SELECT on the icon. To extend this with menus and an ADJUST clik, use
 * the canvas system to attach menu and click handlers to the iconbar canvas.
 */

#ifndef __rov_baricon
#define __rov_baricon


#ifdef __cplusplus
extern "C"
{
#endif

/**
 * Definition of the click handler.
 * @param icon_number the WIMP's icon number for this icon.
 * @see baricon()
 */
typedef void (* baricon_clickp)(int icon_number);

/**
 * Create an icon on the icon bar.
 * @param sprite_name the name of the sprite (must be in the WIMP's sprite
 * area).
 * @param handler the function to call when the user clicks SELECT on the
 * icon.
 * @param ... an optional list of extra parameters, the first of which is a
 * 'flag' parameter, taken from the list below. Where an icon is created in
 * relation to another icon, then the icon's handle, or priority is passed
 * as the fourth parameter. If no third parameter is passed, then the icon
 * is created on the right hand side.
 * @return the icon number for the created icon.
 * @see baricon_clickp()
 * @see BARICON_RIGHT
 * @see BARICON_LEFT
 * @see BARICON_RIGHTICON
 * @see BARICON_LEFTICON
 * @see BARICON_LEFTPRIORITY
 * @see BARICON_RIGHTPRIORITY
 * @see BARICON_LEFTPRIORITYSCANLEFT
 * @see BARICON_LEFTPRIORITYSCANRIGHT
 * @see BARICON_RIGHTPRIORITYSCANLEFT
 * @see BARICON_RIGHTPRIORITYSCANRIGHT
 */
int baricon(char *sprite_name,
	    baricon_clickp handler,
	    ...);

/**
 * Create an icon to the right (default).
 */
#define BARICON_RIGHT (0x3157454e)

/**
 * Create an icon to the left.
 */
#define BARICON_LEFT (0x3257454e)

/**
 * Create an icon to the right of another icon.
 */
#define BARICON_RIGHTICON (0x3357454e)

/**
 * Create an icon to the left of another icon.
 */
#define BARICON_LEFTICON (0x3457454e)

/**
 * Create an icon on the left with priority (scanning left).
 */
#define BARICON_LEFTPRIORITY (0x3557454e)

/**
 * Create an icon on the right with priority (scanning right).
 */
#define BARICON_RIGHTPRIORITY (0x3857454e)

/**
 * Create an icon to the left with priority scanning left.
 */
#define BARICON_LEFTPRIORITYSCANLEFT (0x3557454e)

/**
 * Create an icon to the left with priority scanning right.
 */
#define BARICON_LEFTPRIORITYSCANRIGHT (0x3657454e)

/**
 * Create an icon to the right with priority scanning left.
 */
#define BARICON_RIGHTPRIORITYSCANLEFT (0x3757454e)

/**
 * Create an icon to the right with priority scanning right.
 */
#define BARICON_RIGHTPRIORITYSCANRIGHT (0x3857454e)


#ifdef __cplusplus
}
#endif

#endif
