/**
 * Introduction
 *
 * @title ROV - Easy applications from C
 *
 * @copyright Jason Tribbeck / 7th ARM 1994-2001
 *
 * @authors Jason Tribbeck
 *
 * @file file.h
 *
 * @type helper
 *
 * @description
 *
 * The file helper performs a large number of standard file-based
 * functions, such as loading, saving, creating directories and
 * so forth.
 */

#ifndef __rov_file
#define __rov_file


#ifdef __cplusplus
extern "C"
{
#endif

/**
 * Get the file size.
 * @param file_name the name of the file.
 * @return the size of the file, or -1 if size could not be
 * determined (file not found, for example).
 * @see file_getexecaddr()
 * @see file_getloadaddr()
 * @see file_gettype()
 * @see file_getattr()
 * @see file_getdate()
 * @see file_sizetostr()
 */
int file_size(char *file_name);

/**
 * Check the existence of a directory.
 * @param dir_name the name of the directory.
 * @return non-zero if the directory exists.
 */
int file_direxists(char *dir_name);

/**
 * Synonymn for file_direxists.
 * @mparam n the name of the directory to check.
 * @see file_direxists()
 */
#define file_directoryexists(n) file_direxists(n)

/**
 * Load a file into memory.
 * @param file_name the name of the file.
 * @param data_ptr the pointer to where the file should be loaded.
 * @return the length of the file loaded, or -1 if the file could
 * not be loaded.
 * @note data_ptr must point to an area of memory that has sufficient
 * space to read the file.
 */
int file_load(char *file_name,
	      void *data_ptr);

/**
 * Save a file with a file type.
 * @param file_name the name of the file to save.
 * @param data_ptr the pointer to where the file is stored in
 * memory.
 * @param data_size the size of the file to save.
 * @param file_type the file type to save as.
 * @return an error block if an error occurred, or NULL if successful.
 * @see file_save()
 * @see file_savewithdate()
 */
_kernel_oserror * file_savetype(char *file_name,
				void *data_ptr,
				int data_size,
				int file_type);

/**
 * Save a file with a given load/exec address
 * @param file_name the name of the file to save.
 * @param data_ptr the pointer to where the file is stored in
 * memory.
 * @param data_size the size of the file to save.
 * @param load_addr the load address of the file.
 * @param exec_addr the execution address of the file.
 * @return an error block if an error occurred, or NULL if successful.
 * @see file_savetype()
 * @see file_savewithdate()
 */
_kernel_oserror * file_save(char *file_name,
			    void *data_ptr,
			    int data_size,
			    int load_addr,
			    int exec_addr);

/**
 * Save a file with a file type and a certain date.
 * @param file_name the name of the file to save.
 * @param data_ptr the pointer to where the file is stored in
 * memory.
 * @param data_size the size of the file to save.
 * @param file_type the file type to save as.
 * @param date the date the file should be saved as.
 * @return an error block if an error occurred, or NULL if successful.
 * @see file_save()
 * @see file_savewithdate()
 * @see file_getcurrentdate()
 */
_kernel_oserror * file_savewithdate(char *file_name,
				    void *data_ptr,
				    int data_size,
				    int file_type,
				    date_str *date);

/**
 * Get the type of a file
 * @param file_name the name of the file.
 * @return the file type (or 0x1000 for a directory, 0x2000 for an
 * application, or -1 for unable to get the type).
 * @see file_getexecaddr()
 * @see file_getloadaddr()
 * @see file_size()
 * @see file_getattr()
 * @see file_getdate()
 * @see file_settype()
 */
int file_gettype(char *file_name);

/**
 * Get the load address of a file.
 * @param file_name the name of the file.
 * @return the load address (or -1 if unable to get the address).
 * @see file_gettype()
 * @see file_getexecaddr()
 * @see file_size()
 * @see file_getattr()
 * @see file_getdate()
 * @see file_getdatefromloadexec()
 * @see file_typetostr()
 */
int file_getloadaddr(char *file_name);

/**
 * Get the execution address of a file.
 * @param file_name the name of the file.
 * @return the load address (or -1 if unable to get the address).
 * @see file_gettype()
 * @see file_getloadaddr()
 * @see file_size()
 * @see file_getattr()
 * @see file_getdate()
 * @see file_getdatefromloadexec()
 */
int file_getexecaddr(char *file_name);

/**
 * Synonym for file_size()
 * @see file_size()
 */
#define file_getsize(n) file_size(n)

/**
 * Get the file attributes.
 * @param file_name the name of the file.
 * @return the file attributes
 * @see file_gettype()
 * @see file_getloadaddr()
 * @see file_getexecaddr()
 * @see file_getsize()
 * @see file_getdate()
 * @see file_copyattributes()
 */
int file_getattr(char *file_name);

/**
 * Get the date of a file.
 * @param file_name the name of the file.
 * @param date the memory block to hold the date.
 * @see file_gettype()
 * @see file_getloadaddr()
 * @see file_getexecaddr()
 * @see file_getsize()
 * @see file_getattr()
 * @see file_getcurrentdate()
 * @see file_datecmp()
 * @see file_getdatefromloadexec()
 * @see file_datetostr()
 */
void file_getdate(char *file_name,
		  date_str *date);

/**
 * Get the current date
 * @param date the memory block to hold the date.
 * @see file_datecmp()
 */
void file_getcurrentdate(date_str *date);

/**
 * Sets the type of a file.
 * @param file_name the file name.
 * @param file_type the file type to set it to.
 * @return non-zero for success.
 * @see file_gettype()
 */
BOOL file_settype(char *file_name, 
		  int file_type);

/**
 * Create a directory.
 * @param dir_name the directory name to create.
 * @return non-zero for success.
 * @see file_create()
 */
BOOL file_createdirectory(char *dir_name);

/**
 * Create a file.
 * @param file_name the file name to create.
 * @param file_size the size of the file to create.
 * @return non-zero for success.
 */
BOOL file_create(char *file_name,
		 int file_size);

/**
 * Copies attributes from one file to another.
 * @param dst_name the destination file name.
 * @param src_name the source file name.
 */
void file_copyattributes(char *dst_name,
			 char *src_name);

/**
 * Converts a date into a string.
 * @param date the pointer to the date block.
 * @return a pointer to the string date.
 * @note The string must be copied to another memory area before
 * this function can be called again.
 */
char *file_datetostr(date_str *date);

/**
 * Converts a file type into a string.
 * @param type the file type to convert.
 * @return a pointer to the string type.
 * @note The string must be copied to another memory area before
 * this function can be called again.
 */
char *file_typetostr(int type);

/**
 * Converts a file size into a string.
 * @param size the file size to convert.
 * @return a pointer to the string size.
 * @note The string must be copied to another memory area before
 * this function can be called again.
 */
char *file_sizetostr(int size);

/**
 * Gets the leafname of the file.
 * @param file_name the full file name.
 * @return a pointer to the leaf portion of the full file name.
 * @note The pointer returned is always inside the string that
 * was passed into this function.
 */
char *file_leaf(char *file_name);

/**
 * Work out the date from a given load and execution address.
 * @param load_addr the load address.
 * @param exec_addr the execution address.
 * @param date the memory block to hold the converted date.
 */
void file_getdatefromloadexec(int load_addr,
			      int exec_addr,
			      date_str *date);

/**
 * Compare two dates
 * @param date1 the pointer to the first date.
 * @param date2 the pointer to the second date.
 * @return positive if date1 < date2, 0 if date1 = date 2, or
 * negative if date1 > date2.
 */
int file_datecmp(date_str *date1,
		 date_str *date2);

/**
 * Set the load address, execution address and attributes of
 * a file.
 * @param file_name the name of the file.
 * @param load_addr the load address.
 * @param exec_addr the execution address.
 * @param attr the file attributes
 * @return non-zero if successful.
 */
int file_setloadexecattr(char *file_name,
			 int load_addr,
			 int exec_addr,
			 int attr);


#ifdef __cplusplus
}
#endif

#endif
